# SPDX-FileCopyrightText: 2021 Helmholtz-Zentrum für Umweltforschung GmbH - UFZ
#
# SPDX-License-Identifier: GPL-3.0-or-later
import numpy as np
import pytest

import saqc.lib.ts_operators as tsops
import pandas as pd
from pandas.testing import assert_series_equal


def test_butterFilter():
    assert (
        tsops.butterFilter(pd.Series([1, -1] * 100), cutoff=0.1)
        - pd.Series([1, -1] * 100)
    ).mean() < 0.5


T = True
F = False


@pytest.mark.parametrize(
    "arr,maxc,expected",
    [
        (np.array([]), 1, False),
        (np.array([F]), 1, False),
        (np.array([F, F, F]), 1, False),
        #
        (np.array([T]), 0, True),
        (np.array([T]), 1, False),
        #
        (np.array([F, T, F]), 0, True),
        (np.array([F, T, F]), 1, False),
        #
        (np.array([F, T, T, T, T, F]), 0, True),
        (np.array([F, T, T, T, T, F]), 1, True),
        (np.array([F, T, T, T, T, F]), 2, True),
        (np.array([F, T, T, T, T, F]), 3, True),
        (np.array([F, T, T, T, T, F]), 4, False),
        (np.array([F, T, T, T, T, F]), 5, False),
        #
        (np.array([F, T, T, F, T, T, F]), 2, False),
    ],
)
def test__exceedConsecutiveNanLimit(arr, maxc, expected):
    result = tsops._exceedConsecutiveNanLimit(arr, maxc)
    assert result is expected


def dtSeries(data, freq="1d"):
    index = pd.date_range(start="2020", periods=len(data), freq=freq)
    return pd.Series(data=data, index=index, dtype=float)


@pytest.mark.parametrize(
    "func,data,expected",
    [
        ("identity", dtSeries([1, 2]), dtSeries([1, 2])),
        ("count", dtSeries([0, 0]), dtSeries([2])),
        pytest.param(
            "first",
            dtSeries([1, 2]),
            dtSeries([1, 1]),
            marks=pytest.mark.xfail(reason="BUG (the inner ts.first need an argument)"),
        ),
        pytest.param(
            "last",
            dtSeries([1, 2]),
            dtSeries([1, 1]),
            marks=pytest.mark.xfail(reason="BUG (the inner ts.last need an argument)"),
        ),
        (
            "zeroLog",
            dtSeries([1, 2, np.inf, np.nan]),
            dtSeries([np.log(1), np.log(2), np.inf, np.nan]),
        ),
        pytest.param(
            "zeroLog",
            dtSeries(
                [
                    # 0,
                    -2,
                    -1,
                    -np.inf,
                ]
            ),
            dtSeries([np.nan, np.nan, np.nan]),
            marks=pytest.mark.xfail(reason="zeroLog(0) did not return NaN"),
        ),
    ],
)
def test_tsop_functions(func, data, expected):
    f = getattr(tsops, func)

    resampler = data.resample("2d")

    result = resampler.apply(f)
    assert isinstance(result, pd.Series)

    print()
    print(result)
    print()
    print(expected)
    assert_series_equal(
        result, expected, check_names=False, check_freq=False, check_dtype=False
    )
